module Sis
  module Core
    class Applicant < ApplicationRecord
      validates :first_name, :middle_name, :last_name, :email, presence: true
      validates :email, uniqueness: true

      has_many :applications
      has_one_attached :photo

      has_secure_password

      def full_name
        "#{first_name} #{middle_name} #{last_name}"
      end

      scope :by_application_setup, (lambda do |app_setup_id|
        joins(applications: :application_setup).where('sis_core_applications.application_setup_id': app_setup_id)
      end)

      scope :by_status, (lambda do |status|
        joins(:applications).where('sis_core_applications.status': status)
      end)
    end
  end
end
